@@ -88,6 +88,7 @@ class TotalConnectClient:
         self._valid_credentials = (
             None  # None at start, True after login, False if login fails
         )
+        self._populated = False
         self._module_flags = None
         self._user = None
         self.locations = {}
@@ -146,7 +147,7 @@ class TotalConnectClient:
                     f"total-connect-client invalid session (attempt number {attempts})."
                 )
                 self.token = False
-                self.authenticate_user_login()
+                self.authenticate()
                 return self.request(request, attempts)
             if response.ResultCode == self.CONNECTION_ERROR:
                 logging.debug(
@@ -181,22 +182,33 @@ class TotalConnectClient:
 
     def authenticate(self):
         """Login to the system and populate details.  Return true if successful."""
+        start_time = time.time()
         if self._valid_credentials is not False:
-            start_time = time.time()
-            response = self.request(
-                "LoginAndGetSessionDetails(self.username, self.password, "
-                "self.applicationId, self.applicationVersion)"
-            )
+
+            if self._populated:
+                response = self.request(
+                    "AuthenticateUserLogin(self.username, self.password, "
+                    "self.applicationId, self.applicationVersion)"
+                )
+            else:
+                # this request is very slow, so only use it when necessary
+                response = self.request(
+                    "LoginAndGetSessionDetails(self.username, self.password, "
+                    "self.applicationId, self.applicationVersion)"
+                )
 
             if response["ResultCode"] == self.SUCCESS:
                 logging.debug("Login Successful")
                 self.token = response["SessionID"]
                 self._valid_credentials = True
-                self.populate_details(response)
+                if not self._populated:
+                    self.populate_details(response)
+                    self._populated = True
                 self.times["authenticate()"] = time.time() - start_time
                 return True
 
             self._valid_credentials = False
+            self.token = False
             logging.error(
                 f"Unable to authenticate with Total Connect. ResultCode: "
                 f"{response['ResultCode']}. ResultData: {response['ResultData']}"
@@ -208,35 +220,6 @@ class TotalConnectClient:
         self.times["authenticate()"] = time.time() - start_time
         return False
 
-    def authenticate_user_login(self):
-        """Login to the system.  Return true if successful."""
-        if self._valid_credentials is not False:
-            start_time = time.time()
-            response = self.request(
-                "AuthenticateUserLogin(self.username, self.password, "
-                "self.applicationId, self.applicationVersion)"
-            )
-
-            if response["ResultCode"] == self.SUCCESS:
-                logging.debug("Login Successful")
-                self.token = response["SessionID"]
-                self._valid_credentials = True
-                self.times["authenticate_user_login()"] = time.time() - start_time
-                return True
-
-            self._valid_credentials = False
-            logging.error(
-                f"Unable to authenticate with Total Connect. "
-                f"ResultCode: {response['ResultCode']}. "
-                f"ResultData: {response['ResultData']}"
-            )
-
-        logging.debug(
-            "total-connect-client attempting login with known bad credentials."
-        )
-        self.times["authenticate_user_login()"] = time.time() - start_time
-        return False
-
     def validate_usercode(self, device_id, usercode):
         """Return true if the usercode is valid for the device."""
         response = self.request(
@@ -487,38 +470,25 @@ class TotalConnectClient:
         return False
 
     def zone_bypass(self, zone_id, location_id):
-        """Bypass a zone."""
-        response = self.soapClient.service.Bypass(
-            self.token,
-            location_id,
-            self.locations[location_id].security_device_id,
-            zone_id,
-            f"'{self.locations[location_id].usercode}'",
+        """Bypass a zone.  Return true if successful."""
+        result = self.request(
+            f"Bypass(self.token, "
+            f"{location_id}, "
+            f"{self.locations[location_id].security_device_id}, "
+            f"{zone_id}, "
+            f"'{self.locations[location_id].usercode}')"
         )
 
-        if response.ResultCode == self.INVALID_SESSION:
-            self.authenticate()
-            response = self.soapClient.service.Bypass(
-                self.token,
-                location_id,
-                self.locations[location_id].security_device_id,
-                zone_id,
-                f"'{self.locations[location_id].usercode}'",
-            )
-
-        logging.info(f"Bypass Result Code: {response.ResultCode}.")
-
-        if response.ResultCode == ZONE_BYPASS_SUCCESS:
+        if result["ResultCode"] is ZONE_BYPASS_SUCCESS:
             self.locations[location_id].zones[zone_id].bypass()
-        else:
-            raise Exception(
-                "Could not bypass zone. ResultCode: "
-                + str(response.ResultCode)
-                + ". ResultData: "
-                + str(response.ResultData)
-            )
+            return True
 
-        return self.SUCCESS
+        logging.error(
+            f"Could not bypass zone {zone_id} at location {location_id}."
+            f"ResultCode: {result['ResultCode']}. "
+            f"ResultData: {result['ResultData']}"
+        )
+        return False
 
     def get_zone_details(self, location_id):
         """Get Zone details. Return True if successful."""
@@ -555,7 +525,7 @@ class TotalConnectClient:
             f"ResultCode: {result['ResultCode']}. ResultData: {result['ResultData']}."
         )
         self.times[f"get_zone_details({location_id})"] = time.time() - start_time
-        return True
+        return False
 
 
 class TotalConnectLocation:
